=== modified file 'NEWS'
--- NEWS	2007-04-04 14:05:24 +0000
+++ NEWS	2007-04-05 07:31:43 +0000
@@ -25,6 +25,18 @@
     * Convenience method TestCase.expectFailure ensures that known failures
       do not silently pass.  (Aaron Bentley)
 
+    * New SmartServer hooks facility. There are two initial hooks documented
+      in bzrlib.transport.smart.SmartServerHooks. The two initial hooks allow
+      plugins to execute code upon server startup and shutdown.
+      (Robert Collins).
+
+    * SmartServer in standalone mode will now close its listening socket
+      when it stops, rather than waiting for garbage collection. This primarily
+      fixes test suite hangs when a test tries to connect to a shutdown server.
+      It may also help improve behaviour when dealing with a server running
+      on a specific port (rather than dynamically assigned ports).
+      (Robert Collins)
+ 
   BUGFIXES:
 
     * Don't fail bundle selftest if email has 'two' embedded.  

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2007-03-28 23:28:21 +0000
+++ bzrlib/branch.py	2007-04-04 05:19:38 +0000
@@ -54,6 +54,7 @@
                            NotBranchError, UninitializableFormat,
                            UnlistableStore, UnlistableBranch,
                            )
+from bzrlib.hooks import Hooks
 from bzrlib.symbol_versioning import (deprecated_function,
                                       deprecated_method,
                                       DEPRECATED_PARAMETER,
@@ -892,7 +893,7 @@
             control_files.unlock()
 
 
-class BranchHooks(dict):
+class BranchHooks(Hooks):
     """A dictionary mapping hook name to a list of callables for branch hooks.
     
     e.g. ['set_rh'] Is the list of items to be called when the
@@ -905,7 +906,7 @@
         These are all empty initially, because by default nothing should get
         notified.
         """
-        dict.__init__(self)
+        Hooks.__init__(self)
         # Introduced in 0.15:
         # invoked whenever the revision history has been set
         # with set_revision_history. The api signature is
@@ -944,20 +945,6 @@
         # and an empty branch recieves new_revno of 0, new_revid of None.
         self['post_uncommit'] = []
 
-    def install_hook(self, hook_name, a_callable):
-        """Install a_callable in to the hook hook_name.
-
-        :param hook_name: A hook name. See the __init__ method of BranchHooks
-            for the complete list of hooks.
-        :param a_callable: The callable to be invoked when the hook triggers.
-            The exact signature will depend on the hook - see the __init__ 
-            method of BranchHooks for details on each hook.
-        """
-        try:
-            self[hook_name].append(a_callable)
-        except KeyError:
-            raise errors.UnknownHook('branch', hook_name)
-
 
 # install the default hooks into the Branch class.
 Branch.hooks = BranchHooks()

=== added file 'bzrlib/hooks.py'
--- bzrlib/hooks.py	1970-01-01 00:00:00 +0000
+++ bzrlib/hooks.py	2007-03-25 08:59:56 +0000
@@ -0,0 +1,46 @@
+# Copyright (C) 2007 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Support for plugin hooking logic."""
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+from bzrlib import (
+        errors,
+        )
+""")
+
+
+class Hooks(dict):
+    """A dictionary mapping hook name to a list of callables.
+    
+    e.g. ['FOO'] Is the list of items to be called when the
+    FOO hook is triggered.
+    """
+
+    def install_hook(self, hook_name, a_callable):
+        """Install a_callable in to the hook hook_name.
+
+        :param hook_name: A hook name. See the __init__ method of BranchHooks
+            for the complete list of hooks.
+        :param a_callable: The callable to be invoked when the hook triggers.
+            The exact signature will depend on the hook - see the __init__ 
+            method of BranchHooks for details on each hook.
+        """
+        try:
+            self[hook_name].append(a_callable)
+        except KeyError:
+            raise errors.UnknownHook(self.__class__.__name__, hook_name)

=== modified file 'bzrlib/smart/server.py'
--- bzrlib/smart/server.py	2007-04-09 04:49:55 +0000
+++ bzrlib/smart/server.py	2007-04-10 05:49:02 +0000
@@ -16,10 +16,12 @@
 
 """Server for smart-server protocol."""
 
+import errno
 import socket
 import os
 import threading
 
+from bzrlib.hooks import Hooks
 from bzrlib.smart import medium
 from bzrlib import (
     trace,
@@ -30,7 +32,10 @@
 
 
 class SmartTCPServer(object):
-    """Listens on a TCP socket and accepts connections from smart clients"""
+    """Listens on a TCP socket and accepts connections from smart clients
+
+    hooks: An instance of SmartServerHooks.
+    """
 
     def __init__(self, backing_transport, host='127.0.0.1', port=0):
         """Construct a new server.
@@ -41,36 +46,67 @@
         :param host: Name of the interface to listen on.
         :param port: TCP port to listen on, or 0 to allocate a transient port.
         """
+        # let connections timeout so that we get a chance to terminate
+        # Keep a reference to the exceptions we want to catch because the socket
+        # module's globals get set to None during interpreter shutdown.
+        from socket import timeout as socket_timeout
+        from socket import error as socket_error
+        self._socket_error = socket_error
+        self._socket_timeout = socket_timeout
         self._server_socket = socket.socket()
         self._server_socket.bind((host, port))
-        self.port = self._server_socket.getsockname()[1]
+        self._sockname = self._server_socket.getsockname()
+        self.port = self._sockname[1]
         self._server_socket.listen(1)
         self._server_socket.settimeout(1)
         self.backing_transport = backing_transport
+        self._started = threading.Event()
+        self._stopped = threading.Event()
 
     def serve(self):
-        # let connections timeout so that we get a chance to terminate
-        # Keep a reference to the exceptions we want to catch because the socket
-        # module's globals get set to None during interpreter shutdown.
-        from socket import timeout as socket_timeout
-        from socket import error as socket_error
         self._should_terminate = False
-        while not self._should_terminate:
-            try:
-                self.accept_and_serve()
-            except socket_timeout:
-                # just check if we're asked to stop
-                pass
-            except socket_error, e:
-                trace.warning("client disconnected: %s", e)
-                pass
+        for hook in SmartTCPServer.hooks['server_started']:
+            hook(self.backing_transport.base, self.get_url())
+        self._started.set()
+        try:
+            try:
+                while not self._should_terminate:
+                    try:
+                        conn, client_addr = self._server_socket.accept()
+                    except self._socket_timeout:
+                        # just check if we're asked to stop
+                        pass
+                    except self._socket_error, e:
+                        # if the socket is closed by stop_background_thread
+                        # we might get a EBADF here, any other socket errors
+                        # should get logged.
+                        if e.args[0] != errno.EBADF:
+                            trace.warning("listening socket error: %s", e)
+                    else:
+                        self.serve_conn(conn)
+            except KeyboardInterrupt:
+                # dont log when CTRL-C'd.
+                raise
+            except Exception, e:
+                trace.error("Unhandled smart server error.")
+                trace.log_exception_quietly()
+                raise
+        finally:
+            self._stopped.set()
+            try:
+                # ensure the server socket is closed.
+                self._server_socket.close()
+            except self._socket_error:
+                # ignore errors on close
+                pass
+            for hook in SmartTCPServer.hooks['server_stopped']:
+                hook(self.backing_transport.base, self.get_url())
 
     def get_url(self):
         """Return the url of the server"""
-        return "bzr://%s:%d/" % self._server_socket.getsockname()
+        return "bzr://%s:%d/" % self._sockname
 
-    def accept_and_serve(self):
-        conn, client_addr = self._server_socket.accept()
+    def serve_conn(self, conn):
         # For WIN32, where the timeout value from the listening socket
         # propogates to the newly accepted socket.
         conn.setblocking(True)
@@ -81,20 +117,60 @@
         connection_thread.start()
 
     def start_background_thread(self):
+        self._started.clear()
         self._server_thread = threading.Thread(None,
                 self.serve,
                 name='server-' + self.get_url())
         self._server_thread.setDaemon(True)
         self._server_thread.start()
+        self._started.wait()
 
     def stop_background_thread(self):
+        self._stopped.clear()
+        # tell the main loop to quit on the next iteration.
         self._should_terminate = True
-        # At one point we would wait to join the threads here, but it looks
-        # like they don't actually exit.  So now we just leave them running
-        # and expect to terminate the process. -- mbp 20070215
-        # self._server_socket.close()
-        ## sys.stderr.write("waiting for server thread to finish...")
-        ## self._server_thread.join()
+        # close the socket - gives error to connections from here on in,
+        # rather than a connection reset error to connections made during
+        # the period between setting _should_terminate = True and 
+        # the current request completing/aborting. It may also break out the
+        # main loop if it was currently in accept() (on some platforms).
+        try:
+            self._server_socket.close()
+        except self._socket_error:
+            # ignore errors on close
+            pass
+        if not self._stopped.isSet():
+            # server has not stopped (though it may be stopping)
+            # its likely in accept(), so give it a connection
+            temp_socket = socket.socket()
+            temp_socket.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
+            if not temp_socket.connect_ex(self._sockname):
+                # and close it immediately: we dont choose to send any requests.
+                temp_socket.close()
+        self._stopped.wait()
+        self._server_thread.join()
+
+
+class SmartServerHooks(Hooks):
+    """Hooks for the smart server."""
+
+    def __init__(self):
+        """Create the default hooks.
+
+        These are all empty initially, because by default nothing should get
+        notified.
+        """
+        Hooks.__init__(self)
+        # Introduced in 0.16:
+        # invoked whenever the server starts serving a directory.
+        # The api signature is (backing url, public url).
+        self['server_started'] = []
+        # Introduced in 0.16:
+        # invoked whenever the server stops serving a directory.
+        # The api signature is (backing url, public url).
+        self['server_stopped'] = []
+
+SmartTCPServer.hooks = SmartServerHooks()
 
 
 class SmartTCPServer_for_testing(SmartTCPServer):

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2007-04-04 01:15:02 +0000
+++ bzrlib/tests/__init__.py	2007-04-10 05:49:02 +0000
@@ -755,7 +755,10 @@
         self._benchcalls = []
         self._benchtime = None
         # prevent hooks affecting tests
-        self._preserved_hooks = bzrlib.branch.Branch.hooks
+        self._preserved_hooks = {
+            bzrlib.branch.Branch:bzrlib.branch.Branch.hooks,
+            bzrlib.smart.server.SmartTCPServer:bzrlib.smart.server.SmartTCPServer.hooks,
+            }
         self.addCleanup(self._restoreHooks)
         # this list of hooks must be kept in sync with the defaults
         # in branch.py
@@ -1098,7 +1101,8 @@
             osutils.set_or_unset_env(name, value)
 
     def _restoreHooks(self):
-        bzrlib.branch.Branch.hooks = self._preserved_hooks
+        for klass, hooks in self._preserved_hooks.items():
+            setattr(klass, 'hooks', hooks)
 
     def knownFailure(self, reason):
         """This test has failed for some known reason."""

=== modified file 'bzrlib/tests/test_branch.py'
--- bzrlib/tests/test_branch.py	2007-03-22 19:41:59 +0000
+++ bzrlib/tests/test_branch.py	2007-03-27 21:40:48 +0000
@@ -313,7 +313,7 @@
     def test_installed_hooks_are_BranchHooks(self):
         """The installed hooks object should be a BranchHooks."""
         # the installed hooks are saved in self._preserved_hooks.
-        self.assertIsInstance(self._preserved_hooks, BranchHooks)
+        self.assertIsInstance(self._preserved_hooks[_mod_branch.Branch], BranchHooks)
 
     def test_install_hook_raises_unknown_hook(self):
         """install_hook should raise UnknownHook if a hook is unknown."""

=== modified file 'bzrlib/tests/test_selftest.py'
--- bzrlib/tests/test_selftest.py	2007-04-03 23:38:27 +0000
+++ bzrlib/tests/test_selftest.py	2007-04-10 05:49:02 +0000
@@ -1162,6 +1162,8 @@
         """The bzrlib hooks should be sanitised by setUp."""
         self.assertEqual(bzrlib.branch.BranchHooks(),
             bzrlib.branch.Branch.hooks)
+        self.assertEqual(bzrlib.smart.server.SmartServerHooks(),
+            bzrlib.smart.server.SmartTCPServer.hooks)
 
     def test__gather_lsprof_in_benchmarks(self):
         """When _gather_lsprof_in_benchmarks is on, accumulate profile data.

=== modified file 'bzrlib/tests/test_smart_transport.py'
--- bzrlib/tests/test_smart_transport.py	2007-04-10 02:09:14 +0000
+++ bzrlib/tests/test_smart_transport.py	2007-04-10 05:49:02 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -781,6 +781,7 @@
     def test_get_error_unexpected(self):
         """Error reported by server with no specific representation"""
         class FlakyTransport(object):
+            base = 'a_url'
             def get_bytes(self, path):
                 raise Exception("some random exception from inside server")
         smart_server = server.SmartTCPServer(backing_transport=FlakyTransport())
@@ -818,14 +819,38 @@
         self.server = server.SmartTCPServer(self.backing_transport)
         self.server.start_background_thread()
         self.transport = remote.SmartTCPTransport(self.server.get_url())
+        self.addCleanup(self.tearDownServer)
 
-    def tearDown(self):
+    def tearDownServer(self):
         if getattr(self, 'transport', None):
             self.transport.disconnect()
+            del self.transport
         if getattr(self, 'server', None):
             self.server.stop_background_thread()
-        super(SmartTCPTests, self).tearDown()
-        
+            del self.server
+
+
+class TestServerSocketUsage(SmartTCPTests):
+
+    def test_server_setup_teardown(self):
+        """It should be safe to teardown the server with no requests."""
+        self.setUpServer()
+        server = self.server
+        transport = remote.SmartTCPTransport(self.server.get_url())
+        self.tearDownServer()
+        self.assertRaises(errors.ConnectionError, transport.has, '.')
+
+    def test_server_closes_listening_sock_on_shutdown_after_request(self):
+        """The server should close its listening socket when it's stopped."""
+        self.setUpServer()
+        server = self.server
+        self.transport.has('.')
+        self.tearDownServer()
+        # if the listening socket has closed, we should get a BADFD error
+        # when connecting, rather than a hang.
+        transport = remote.SmartTCPTransport(server.get_url())
+        self.assertRaises(errors.ConnectionError, transport.has, '.')
+
 
 class WritableEndToEndTests(SmartTCPTests):
     """Client to server tests that require a writable transport."""
@@ -911,7 +936,46 @@
         self.setUpServer(readonly=True)
         self.assertRaises(errors.TransportNotPossible, self.transport.mkdir,
             'foo')
-        
+
+
+class TestServerHooks(SmartTCPTests):
+
+    def capture_server_call(self, backing_url, public_url):
+        """Record a server_started|stopped hook firing."""
+        self.hook_calls.append((backing_url, public_url))
+
+    def test_server_started_hook(self):
+        """The server_started hook fires when the server is started."""
+        self.hook_calls = []
+        server.SmartTCPServer.hooks.install_hook('server_started',
+            self.capture_server_call)
+        self.setUpServer()
+        # at this point, the server will be starting a thread up.
+        # there is no indicator at the moment, so bodge it by doing a request.
+        self.transport.has('.')
+        self.assertEqual([(self.backing_transport.base, self.transport.base)],
+            self.hook_calls)
+
+    def test_server_stopped_hook_simple(self):
+        """The server_stopped hook fires when the server is stopped."""
+        self.hook_calls = []
+        server.SmartTCPServer.hooks.install_hook('server_stopped',
+            self.capture_server_call)
+        self.setUpServer()
+        result = [(self.backing_transport.base, self.transport.base)]
+        # check the stopping message isn't emitted up front.
+        self.assertEqual([], self.hook_calls)
+        # nor after a single message
+        self.transport.has('.')
+        self.assertEqual([], self.hook_calls)
+        # clean up the server
+        self.tearDownServer()
+        # now it should have fired.
+        self.assertEqual(result, self.hook_calls)
+
+# TODO: test that when the server suffers an exception that it calls the
+# server-stopped hook.
+
 
 class SmartServerRequestHandlerTests(tests.TestCaseWithTransport):
     """Test that call directly into the handler logic, bypassing the network."""

=== modified file 'bzrlib/trace.py'
--- bzrlib/trace.py	2007-01-30 10:51:38 +0000
+++ bzrlib/trace.py	2007-03-27 04:57:59 +0000
@@ -181,6 +181,8 @@
 
     The exception string representation is used as the error
     summary, unless msg is given.
+
+    Please see log_exception_quietly() for the replacement API.
     """
     if msg:
         error(msg)

